from izzylib import DotDict, ap_date
from urllib.parse import urlparse
from uuid import uuid4

from .config import config
from .functions import fetch_actor, push_message


class Message(DotDict):
	def __init__(self, name, *args):
		self.update(getattr(self, f'msg_{name}')(*args))


	def send(self, inbox):
		return push_message(inbox, self)


	def msg_accept(self, followid, actor):
		message = {
			'@context': 'https://www.w3.org/ns/activitystreams',
			'type': 'Accept',
			'to': [actor],
			'actor': f'https://{config.host}/actor',

			'object': {
				'type': 'Follow',
				'id': followid,
				'object': f'https://{config.host}/actor',
				'actor': actor
			},

			'id': f'https://{config.host}/activities/{str(uuid4())}',
		}

		return message


	def msg_reject(self, followid, actor):
		message = {
			'@context': 'https://www.w3.org/ns/activitystreams',
			'type': 'Reject',
			'to': [actor],
			'actor': f'https://{config.host}/actor',

			'object': {
				'type': 'Follow',
				'id': followid,
				'object': f'https://{config.host}/actor',
				'actor': actor
			},

			'id': f'https://{config.host}/activities/{str(uuid4())}',
		}

		return message


	def msg_announce(self, object_id):
		data = {
			'@context': 'https://www.w3.org/ns/activitystreams',
			'type': 'Announce',
			'to': [f'https://{config.host}/followers'],
			'actor': f'https://{config.host}/actor',
			'object': object_id,
			'id': f'https://{config.host}/activities/{str(uuid4())}'
		}

		return data


	def msg_create(self, inbox, object_id):
		data = {
			'@context': 'https://www.w3.org/ns/activitystreams',
			'type': 'Create',
			'to': [inbox],
			'actor': f'https://{config.host}/actor',
			'object': object_id,
			'id': f'https://{config.host}/activities/{str(uuid4())}'
		}


	# needs to be redone
	def msg_note(self, handle, actor, message):
		domain = urlparse(user_inbox).netloc
		inbox = fetch_actor(actor)

		data = {
			"@context": "https://www.w3.org/ns/activitystreams",
			"id": f"https://{config.host}/activities/{str(uuid4())}",
			"type": "Create",
			"actor": f"https://{config.host}/actor",
			"object": {
				"id": f"https://{config.host}/activities/{str(uuid4())}",
				"type": "Note",
				"published": ap_date(),
				"attributedTo": f"https://{config.host}/actor",
				"content": message,
				'to': [inbox],
				'tag': [{
					'type': 'Mention',
					'href': actor,
					'name': f'@{handle}@{domain}'
				}],
			}
		}

		return data
